/* main.c */

#ifdef _WIN32
#define _CRT_SECURE_NO_WARNINGS
#endif

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#ifdef _WIN32
#include <windows.h>
#include <conio.h>
#endif
#include "locksim.h"

char version[] = "0.4.1 (2021)";

/*--------------------------------------------------------*/

int verbosity = -1;

/*--------------------------------------------------------*/

void usage(void) {
    fprintf(stderr, "Usage: locksim  [ -V ]  [ -v VERBOSITY ]  [ -s SEED ]  [ -r REPS ] SCENARIO-FILE\n");
#ifdef _WIN32
    fprintf(stderr, "Press a key to exit Locksim!\n");
    _getch();
#endif
    exit(EXIT_FAILURE);
}

int main(int argc, char *argv[]) {
    char *file_name = NULL;
    int reps = -1;
    int seed = -1;
    int argnr = 1;

    while (argnr < argc) {
        if (argv[argnr][0] == '-') {
            if (strlen(argv[argnr]) != 2) {
                usage();
            }
            else if (argv[argnr][1] == 'V') {
                ++argnr;
                printf("Locksim version %s\n", version);
                exit(EXIT_SUCCESS);
            }
            else if (argv[argnr][1] == 'v' && argnr + 1 < argc) {
                ++argnr;
                if (sscanf(argv[argnr], "%d", &verbosity) != 1 || verbosity < 0)
                    usage();
            }
            else if (argv[argnr][1] == 's' && argnr + 1 < argc) {
                ++argnr;
                if (seed != -1 || sscanf(argv[argnr], "%d", &seed) != 1 || seed < 0)
                    usage();
            }
            else if (argv[argnr][1] == 'r' && argnr + 1 < argc) {
                ++argnr;
                if (reps != -1 || sscanf(argv[argnr], "%d", &reps) != 1 || reps <= 0)
                    usage();
            }
        }
        else if (file_name == NULL) {
            file_name = argv[argnr];
        }
        else
            usage();
        ++argnr;
    }

    if (file_name == NULL)
        usage();
    if (verbosity < 0)
        verbosity = 1;
    if (reps < 0)
        reps = 1;

    FILE *tsin = fopen(file_name, "r");
    if (tsin == 0) {
        fatal_error("Couldn't read input file '%s'", file_name);
    }

    init_symbols();
    struct TreeNode *s = parse_scenario(tsin);
    fclose(tsin);

    if (seed != -1)
        srand(seed);
    else
        srand((unsigned int)time(NULL));

    for (int i = 0; i < reps; ++i)
        execute_scenario(s, verbosity);
    printf("Tried the scenario in '%s' %d time%s.\n", file_name, reps, reps == 1 ? "" : "s");
    show_results();

#ifdef _WIN32
    fprintf(stderr, "Press a key to exit Locksim!\n");
    _getch();
#endif

} /* main */
