/* scanner.c -- The scanner reads one token from a FILE. Lexical values are put in "lexical_value".  */

#ifdef _WIN32
#define _CRT_SECURE_NO_WARNINGS
#endif

#include <stdio.h>
#include <ctype.h>
#include "locksim.h"

int lexical_value;
int line_number = 1;

int scan(FILE *f) {
    while(1) {
        int c = getc(f);
        if (c == EOF)
            return DONE;
        else if (c == '\n')
            ++line_number;
        else if (c == ' ' || c == '\t' || c == '\r')
            ; /* Just skip other whitespace */
        else if (isdigit(c)) {
            /* This can be the start of a number */
            ungetc(c, f);
            fscanf(f, "%d", &lexical_value);
            return NUMBER;
        }
        else if (isalpha(c) || c == '_') {
            /* This can be the start of an identifier */
            #define MAX_ID_LENGTH 100
            char buffer[MAX_ID_LENGTH + 1];
            int n = 0;
            while (isalnum(c) || c == '_') {
                buffer[n++] = c;
                if (n > MAX_ID_LENGTH)
                    fatal_error("Identifier too long.\n");
                c = getc(f);
            }
            buffer[n] = '\0';
            if (c != EOF)
                ungetc(c, f);
            lexical_value = lookup_or_insert_symbol(buffer, ID);
            // printf("[%s]", symbol_table[lexical_value].name);
            return get_symbol_type(lexical_value);
        }
        else if (c == '/') {
            /* Could be either division or the start a comment */
            int c2 = getc(f);
            if (c2 == '*') {
                int inside_comment = 1;
                while (inside_comment && (c = getc(f)) != EOF) {
                    if (c == '\n')
                        ++line_number;
                    else if (c == '*') {
                        int c3 = getc(f);
                        if (c3 == '\n')
                            ++line_number;
                        else if (c3 == '/')
                            inside_comment = 0;
                    }
                }
                if (c == EOF)
                    fatal_error("Unterminated comment");
            }
            else if (c2 == '/') {
                int inside_comment = 1;
                while (inside_comment && (c = getc(f)) != EOF) {
                    if (c == '\n') {
                        ++line_number;
                        inside_comment = 0;
                    }
                }
            }
            else {
                ungetc(c2, f);
                return c;
            }
        }
        else {
            lexical_value = -1;
            return c;
        }
    }
} /* scan */
